--	LibBuilder-1.0
--	Creates a private "addon table" for use in multi-file libraries.


local MAJOR, MINOR = "LibBuilder-1.0", 1


local LibBuilder = LibStub:NewLibrary(MAJOR, MINOR)
if not LibBuilder then return end


LibBuilder.errorStrings = {
	SHELL = "%s: %s!", 
	BADVERSION = "NewLib requires a textual major and numeric minor version", 
	COLLISION = "Attempted to build %s, rev. %s before closing %s, rev. %s", 
	NOLIB1 = "GetLib called but no library is being built", 
	NOLIB2 = "CloseLib called but no library is being built", 
	LEFTOPEN = "Library %s, rev. %s was not closed properly",
}


--	Create a frame so we can watch for ADDON_LOADED and throw 
--	a warning if a library is still open when it fires.
local frame = CreateFrame("Frame")
frame:Hide()
frame:SetScript("OnEvent", function() LibBuilder:OnEvent() end)
frame:RegisterEvent("ADDON_LOADED")
LibBuilder.frame = frame


--	used fields are listed here for documentation purposes
LibBuilder.workspace = nil
LibBuilder.major = nil
LibBuilder.minor = nil


---	Registers a new library and creates a workspace for it.
--	@param	major		The major version of the library being registered.
--	@param	minor		The minor version of the library being registered.
--	@param	workspace	Optional, an existing table to use as the workspace.
--	@return	workspace	The workspace for the new library.
function LibBuilder:NewLib(major, minor, workspace)
	--	check the supplied version info
	if type(major) ~= "string" or type(minor) ~= "number" then
		local message = self:BuildErrorMessage("BADVERSION", major, minor)
		error(message)
	end
	
	--	check for collisions
	if self.major or self.minor or self.workspace then
		local message = self:BuildErrorMessage("COLLISION", self.major, self.minor, major, minor)
		
		--	we want the user to see the error but we want don't want to break the new lib
		--	TODO: should I change the code to save the old values, run the normal stuff, 
		--	and then use error?
		geterrorhandler()(message)
	end
	
	--	show the frame so that we can use ADDON_LOADED 
	--	to check if this library was not closed.
	self.frame:Show()

	self.major = major
	self.minor = minor
	
	--	if the programmer gave us a funky workspace, that's their problem
	self.workspace = workspace or {}
	
	return self.workspace
end


---	Retrieves the version info and workspace for the current active library.
--	@return major		Major version of the library.
--	@return minor		Minor version of the library.
--	@return workspace	The workspace for the library.
function LibBuilder:GetLib()
	if not self.major then		--	just check major to allow for flexibility
		local message = self:BuildErrorMessage("NOLIB1")
		error(message)
	end
	
	return self.major, self.minor, self.workspace
end


---	Closes the active library. This is used for tracking, to make sure 
--	that two libraries don't get clobbered together.
function LibBuilder:CloseLib()
	--	we don't want programmers calling this to 
	--	suppress errors before they open their lib.
	if not self.major then
		local message = self:BuildErrorMessage("NOLIB2")
		error(message)
	end
	
	self.major, self.minor, self.workspace = nil, nil, nil
end


---	Private, builds an error message
--	@param	errorName	Name of the error
--	@param	...			List of arguments applicable to the error type.
--	@return message		The complete error string.
function LibBuilder:BuildErrorMessage(errorName, ...)
	local body = self.errorStrings[errorName]:format(...)
	local message = self.errorStrings.SHELL:format(MAJOR, body)

	return message
end


function LibBuilder:OnEvent()
	--	we're only registered for ADDON_LOADED
	
	--	NOTE: it doesn't matter -which- addon has just loaded; this 
	--	event should never fire between the opening and closing of a library.

	--	hide frame so we don't get another update unless another library 
	--	is created, and do it now in case we have to throw the error
	self.frame:Hide()

	if self.major then
		local major, minor = self.major, self.minor 
		
		--	close it so the error only happens once
		self:CloseLib()
		
		local message = self:BuildErrorMessage("LEFTOPEN", major, minor)
		error(message)
	end
end
